
(load "psyk.ss")
(load "ircbot.ss")

(define *my-nick* "psyk")
(define *me* (string->symbol *my-nick*))
(define *irc-server* "irc.saunalahti.fi")
(define *irc-port* 6662)
(define *username* "Pekka Sylvi Yves Kortteinen")
(define *characters-per-second* 4.6)

(define bot-say irc-say)
(define bot-join irc-join)
(define bot-reply irc-reply)
(define (bot-quit message)
  (irc-write `(message quit ,message))
  (throw 'toplevel))

;; initialisation

(define (bot-initialise)
  (standard-hooks)
  (load-answers! "answers.db")
  (add-admin '(user atehwa (username "atehwa") (hostname "aulis.sange.fi")))
  (for-each
    (lambda (command)
      (let ((name (car command)) (func (cadr command)))
	(add-admin-command
	  (string->symbol name)
	  (lambda (source dest params)
	    (bot-reply source dest
		       (format #f "~s" (func (string->list params))))))))
    special-command-handlers)
  (add-admin-command
    'fix (lambda (source dest text)
	   (if (string=? text "")
	     (psykbot-show-answer-history source dest)
	     (psykbot-delete-answer source dest (string->number text)))))
  (add-hook 'privmsg 'reply
	    (lambda (source dest text)
	      (if (not (regexp-exec command-regexp text))
		(if (eq? (irc-nick dest) *me*)
		  (process-personal text source source)
		  (process-public text dest source)))))
  (irc-reconnect-loop *irc-server* *irc-port* *me* *username*))

;; answer history

(define (psykbot-show-answer-history source dest)
  (let ((num 0))
    (for-each (lambda (hist)
		(bot-reply source dest
			   (format #f "~s: ~s <- ~s"
				   num (cadr hist) (car hist)))
		(set! num (+ num 1)))
	      (take-answer-history))))

(define (psykbot-delete-answer source dest num)
  (bot-reply source dest (format #f "~s" (delete-answer num))))

;; privmsg handling

(define *bot-contexts* (make-btree))
(define (get-context chan-or-nick)
  (btree-get *bot-contexts* (message-part->string chan-or-nick) '()))
(define (set-context chan-or-nick context)
  (btree-set! *bot-contexts* (message-part->string chan-or-nick) context))

(define (prompt-prefix-length str)
  (let loop ((len 0) (nickchars (string->list *my-nick*)))
    (cond ((and (null? nickchars)
		(memv (string-ref str len) '(#\space #\: #\,)))
	   (loop (+ len 1) '()))
	  ((null? nickchars) len)
	  ((char-ci=? (string-ref str len) (car nickchars))
	   (loop (+ len 1) (cdr nickchars)))
	  (else 0))))

(define (prompted? str)
  (> (prompt-prefix-length str) (string-length *my-nick*)))

(define (process-public message channel nick)
  (if (prompted? message)
    (process-personal message channel nick)
    (let* ((context (make-context message))
	   (answer (get-answer context)))
      (if (>= *last-max-score* *react-threshold*)
	(process-answer answer context channel nick)))))

(define (process-personal message channel nick)
  (let* ((realmsg (substring message (prompt-prefix-length message)))
	 (old-context (get-context channel)))
    (learn! realmsg old-context)
    (let* ((context (update-context (make-context realmsg) old-context))
	   (answer (get-answer context)))
      (process-answer answer context channel nick))))

(define (process-answer answer context destination nick)
  (mark-used! answer)
  (add-timer
    (inexact->exact (round (/ (string-length answer) *characters-per-second*)))
    (lambda () (send-answer answer context destination nick))))

(define (send-answer answer context destination nick)
  (bot-say destination
	   (if (eq? (irc-nick destination) (irc-nick nick)) answer
	     (string-append (symbol->string (irc-nick nick)) ": " answer)))
  (set-context destination (update-context (make-context answer) context)))

; (bot-initialise)

